"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("./jinghuan");

require("./props");

var _path = _interopRequireDefault(require("path"));

var _cluster = _interopRequireDefault(require("cluster"));

var _watcher = _interopRequireDefault(require("./core/watcher"));

var _loaders = _interopRequireDefault(require("./loaders"));

var _jinghuanCluster = _interopRequireDefault(require("jinghuan-cluster"));

var _define = _interopRequireDefault(require("jinghuan-helper/define"));

var _isDirectory = _interopRequireDefault(require("jinghuan-helper/isDirectory"));

var _isArray = _interopRequireDefault(require("lodash/isArray"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _isFunction = _interopRequireDefault(require("lodash/isFunction"));

var _each = _interopRequireDefault(require("lodash/each"));

var _http = _interopRequireDefault(require("http"));

var _https = _interopRequireDefault(require("https"));

var _safe = _interopRequireDefault(require("colors/safe"));

var _socket = _interopRequireDefault(require("./core/socket"));

var _koa = _interopRequireDefault(require("koa"));

var _clearModule = _interopRequireDefault(require("clear-module"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class Application {
  constructor(options = {}) {
    this.options = {};

    this.watcherCallBack = file => {
      if (this.masterInstance) {
        this.masterInstance.forceReloadWorkers();
      } else {
        try {
          let resolveFile = require.resolve(file);

          let module = require.cache[resolveFile];

          if (module) {
            (0, _clearModule.default)(resolveFile);
            jinghuanjs.logger.log(`[Master] Reload ${file}`);
          }
        } catch {}
      }
    };

    this.options = options;
    (0, _define.default)('ROOT_PATH', options.ROOT_PATH, jinghuan);
    (0, _define.default)('APP_PATH', options.APP_PATH, jinghuan);
    (0, _define.default)('JH_PATH', options.JH_PATH, jinghuan);

    if (options.config) {
      (0, _define.default)('CONFIG_PATH', options.config, jinghuan);
    } else {
      (0, _define.default)('CONFIG_PATH', _path.default.join(options.ROOT_PATH, options.source == 'src' ? 'dev-config' : 'config', options.env), jinghuan);
    }

    (0, _define.default)('workers', options.workers, jinghuan);
    (0, _define.default)('source', options.source, jinghuan);
    (0, _define.default)('modules', options.modules, jinghuan);
    (0, _define.default)('env', options.env, jinghuan);
    (0, _define.default)('PORT', options.port || 8409, jinghuan);
    (0, _define.default)('HOST', options.host, jinghuan);
    (0, _define.default)('paths', options.paths, jinghuan);
    (0, _define.default)('watcher', options.watcher, jinghuan);
    (0, _define.default)('socket', options.socket, jinghuan);
    (0, _define.default)('SSL', !!options.ssl, jinghuan);
    (0, _define.default)('LOG_PATH', options.LOG_PATH, jinghuan);
    let modulePaths = {};
    [...options.modules, 'common'].forEach(moduleName => {
      if ((0, _isDirectory.default)(_path.default.join(jinghuanjs.ROOT_PATH, `node_modules/jinghuan-module-${moduleName}`))) {
        modulePaths[moduleName] = _path.default.join(jinghuanjs.ROOT_PATH, `node_modules/jinghuan-module-${moduleName}`);
      } else {
        modulePaths[moduleName] = _path.default.join(options.APP_PATH, moduleName);
      }
    });
    options.modulePaths = modulePaths;
    (0, _define.default)('modulePaths', modulePaths, jinghuan);
    jinghuanjs.registerTextHandler('sql');
  }

  notifier(err) {
    if (!jinghuanjs.notifier) {
      return;
    }

    let notifier = jinghuanjs.notifier;

    if (!(0, _isArray.default)(notifier)) {
      notifier = [notifier];
    }

    notifier[0](Object.assign({
      title: 'JinghuanJs Transpile Error',
      message: err.message
    }, notifier[1]));
  }

  startWatcher() {
    if (!jinghuanjs.watcher) {
      return;
    }

    let srcPath = [_path.default.join(jinghuanjs.ROOT_PATH, 'config', jinghuanjs.env)];
    (0, _each.default)(jinghuanjs.modulePaths, (loadPath, module) => {
      let jhFile = _path.default.join(loadPath, '.jinghuanjs');

      if (_fsExtra.default.pathExistsSync(jhFile)) {
        let conf = _fsExtra.default.readJsonSync(jhFile);

        for (let cpath of conf.paths || {}) {
          srcPath.push(_path.default.join(loadPath, cpath));
        }
      }
    });
    let allowExts = jinghuanjs.config('watcher.allowExts');
    const instance = new _watcher.default({
      srcPath,
      allowExts
    }, fileInfo => {
      this.watcherCallBack(fileInfo);
    });
    instance.watch();
  }

  getMasterInstance() {
    this.masterInstance = new _jinghuanCluster.default.Master({
      port: jinghuanjs.PORT,
      host: jinghuanjs.HOST,
      workers: jinghuanjs.workers
    });
    return this.masterInstance;
  }

  runInMaster(tag = "Master") {
    let lines = [`JinghuanJs  ${jinghuanjs.version}`, `SSL         ${jinghuanjs.SSL}`, `HOST        [${jinghuanjs.HOST}]`, `PORT        ${jinghuanjs.PORT}`, `ROOT_PATH   ${jinghuanjs.ROOT_PATH}`, `APP_PATH    ${jinghuanjs.APP_PATH}`, `CONFIG_PATH ${jinghuanjs.CONFIG_PATH}`, `LOG_PATH    ${jinghuanjs.LOG_PATH}`, `ENVIROMENT  ${jinghuanjs.env}`, `SOURCE      ${jinghuanjs.source}`, `MODULES     [${jinghuanjs.modules}]`, `MIDDLEWARE  [${jinghuanjs.middlewares}]`, `SOCKET      [${jinghuanjs.sockets}]`];
    this.consoleLines(lines);
    let instance = this.getMasterInstance();
    Promise.resolve(instance.startServer()).then(() => {
      jinghuanjs.logger.info(`[${tag}] Start OK`);
      let conf = jinghuanjs.config(tag.toLocaleLowerCase());

      if ((0, _isFunction.default)(conf)) {
        conf();
      }
    });
  }

  getWorkerInstance() {
    let config = jinghuanjs.config('config') || {};
    let server;
    this.workerInstance = new _jinghuanCluster.default.Worker(_objectSpread(_objectSpread({}, config), {}, {
      workers: jinghuanjs.workers,
      createServer: async () => {
        if (this.options.ssl) {
          server = _https.default.createServer(this.options.ssl, jinghuanjs.app.callback()).listen(jinghuanjs.PORT);
        } else {
          server = _http.default.createServer(jinghuanjs.app.callback()).listen(jinghuanjs.PORT);
        }

        (0, _define.default)('server', server, jinghuanjs.app);

        if (this.options.socket) {
          let socket = new _socket.default(jinghuanjs.app.server);
          (0, _define.default)('socket', socket, jinghuanjs.app);
        }

        return server;
      }
    }));
    return this.workerInstance;
  }

  runInWorker(tag = "Worker") {
    let lines = [`JinghuanJs  ${jinghuanjs.version}`, `SSL         ${jinghuanjs.SSL}`, `HOST        [${jinghuanjs.HOST}]`, `PORT        ${jinghuanjs.PORT}`, `ROOT_PATH   ${jinghuanjs.ROOT_PATH}`, `APP_PATH    ${jinghuanjs.APP_PATH}`, `CONFIG_PATH ${jinghuanjs.CONFIG_PATH}`, `LOG_PATH    ${jinghuanjs.LOG_PATH}`, `ENVIROMENT  ${jinghuanjs.env}`, `SOURCE      ${jinghuanjs.source}`, `MODULES     [${jinghuanjs.modules}]`, `MIDDLEWARE  [${jinghuanjs.middlewares}]`, `SOCKET      [${jinghuanjs.sockets}]`];
    this.consoleLines(lines);
    let instance = this.getWorkerInstance();
    Promise.resolve(instance.startServer()).then(() => {
      this.init = true;
      jinghuanjs.logger.info(`[${tag}] Start OK`);
      let conf = jinghuanjs.config(tag.toLocaleLowerCase());

      if ((0, _isFunction.default)(conf)) {
        conf();
      }
    });
  }

  consoleLines(lines) {
    let length = 0;
    lines.forEach(str => {
      length = str.length > length ? str.length : length;
    });

    let s = _safe.default.yellow("║");

    console.log(_safe.default.yellow("╔".padEnd(length + 3, "═") + "╗"));
    lines.forEach(str => {
      console.log(s + " " + str.padEnd(length, " ") + " " + s);
    });
    console.log(_safe.default.yellow("╚".padEnd(length + 3, "═") + "╝"));
  }

  initApp() {
    (0, _define.default)('app', new _koa.default(), jinghuanjs);
  }

  run() {
    const loaders = new _loaders.default();

    try {
      if (_cluster.default.isMaster && jinghuanjs.workers > 0) {
        loaders.load(false);
        this.startWatcher();
        return this.runInMaster();
      } else {
        this.initApp();
        loaders.load(true);

        if (jinghuanjs.workers === 0) {
          this.startWatcher();
        }

        return this.runInWorker(jinghuanjs.workers === 0 ? 'Master' : '');
      }
    } catch (e) {
      console.error(e);
    }
  }

}

;
var _default = Application;
exports.default = _default;
module.exports = exports.default;