"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _debug = _interopRequireDefault(require("debug"));

var _isFunction = _interopRequireDefault(require("lodash/isFunction"));

var _isArray = _interopRequireDefault(require("lodash/isArray"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = (0, _debug.default)(`JH:core/events[${process.pid}]`);
const EVENTS = Symbol('events');

class Events {
  constructor() {
    this[EVENTS] = {};

    this.on = (name, listener) => {
      log(`on ${name}`);

      if (!(0, _isFunction.default)(listener)) {
        throw TypeError('listener must be a function');
      }

      this[EVENTS][name] = this[EVENTS][name] || [];
      this[EVENTS][name].push(listener);
    };

    this.emit = async (name, ...args) => {
      log(`emit ${name}`);
      let listeners = this[EVENTS][name];

      if ((0, _isArray.default)(listeners)) {
        for (let listener of listeners) {
          if ((0, _isFunction.default)(listener)) {
            await listener(...args);
          }
        }
      }
    };

    this.un = (name, listener) => {
      log(`un ${name}`);
      let listeners = this[EVENTS][name];

      if ((0, _isArray.default)(listeners)) {
        for (let i in listeners) {
          if (listeners[i] == listener) {
            delete listeners[i];
          }
        }
      }
    };
  }

  isEvent(name) {
    return this[EVENTS][name] != null && this[EVENTS][name].length > 0;
  }

}

var _default = Events;
exports.default = _default;
module.exports = exports.default;