"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _pathToRegexp = _interopRequireDefault(require("path-to-regexp"));

var _debug = _interopRequireDefault(require("debug"));

var _define = _interopRequireDefault(require("jinghuan-helper/define"));

var _isFunction = _interopRequireDefault(require("lodash/isFunction"));

var _isString = _interopRequireDefault(require("lodash/isString"));

var _utils = require("../../utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const log = (0, _debug.default)(`JH:core/loader/middleware[${process.pid}]`);
const RUN = Symbol('run');

class Middleware {
  constructor() {}

  createRegexp(match) {
    if ((0, _isFunction.default)(match)) {
      return match;
    }

    if (match) {
      return (0, _pathToRegexp.default)(match);
    }
  }

  checkMatch(rule, ctx) {
    if ((0, _isFunction.default)(rule)) {
      return rule(ctx);
    }

    return rule.test(ctx.path);
  }

  async requireMiddlewareFile(item) {
    let middleware = "";
    let commonPath = jinghuanjs.modulePaths['common'];

    let appMid = _path.default.join(commonPath, 'middleware', item.handle);

    let nodeMid = 'jinghuan-middleware-' + item.handle;

    if (_fsExtra.default.pathExistsSync(appMid)) {
      middleware = appMid;
    } else if ((0, _utils.requireResolve)(nodeMid)) {
      middleware = (0, _utils.requireResolve)(nodeMid);
    } else {
      throw new Error(`Middleware failed to load. run npm i ${nodeMid}`);
    }

    let cache = require.cache[middleware];

    if (!cache) {
      let handle = await Promise.resolve(`${middleware}`).then(s => _interopRequireWildcard(require(s)));

      if (handle) {
        return (handle.default || handle)(item.options, jinghuanjs.app);
      }
    }
  }

  parse(middlewares = []) {
    return middlewares.map(item => {
      if ((0, _isString.default)(item)) {
        return {
          handle: item
        };
      }

      return item;
    }).filter(item => {
      return !('enable' in item) || item.enable;
    }).map(item => {
      const match = this.createRegexp(item.match);
      const ignore = this.createRegexp(item.ignore);
      return async (ctx, next) => {
        if (item[RUN] == null) {
          if ((0, _isString.default)(item.handle)) {
            item[RUN] = await this.requireMiddlewareFile(item);
          } else if (!(0, _isFunction.default)(item[RUN])) {
            item[RUN] = item.handle(item.options, jinghuanjs.app);
          }
        }

        if (match && !this.checkMatch(match, ctx) || ignore && this.checkMatch(ignore, ctx)) {
          return next();
        }

        return item[RUN](ctx, next);
      };
    });
  }

  load() {
    let middlewares = jinghuanjs.config('middleware') || [];
    let ms = middlewares.map((v, k) => {
      return v.handle;
    });
    (0, _define.default)('middlewares', ms, jinghuan);
    return this.parse(middlewares);
  }

}

var _default = new Middleware();

exports.default = _default;
module.exports = exports.default;