"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _pathToRegexp = _interopRequireDefault(require("path-to-regexp"));

var _isFile = _interopRequireDefault(require("jinghuan-helper/isFile"));

var _define = _interopRequireDefault(require("jinghuan-helper/define"));

var _isFunction = _interopRequireDefault(require("lodash/isFunction"));

var _isString = _interopRequireDefault(require("lodash/isString"));

var _debug = _interopRequireDefault(require("debug"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = (0, _debug.default)(`JH:core/loader/socket[${process.pid}]`);
const RUN = Symbol('run');

class Middleware {
  constructor() {}

  createRegexp(match) {
    if ((0, _isFunction.default)(match)) {
      return match;
    }

    if (match) {
      return (0, _pathToRegexp.default)(match);
    }
  }

  checkMatch(rule, ctx) {
    if ((0, _isFunction.default)(rule)) {
      return rule(ctx);
    }

    return rule.test(ctx.path);
  }

  checkMid(name) {
    try {
      return require.resolve(name, {
        paths: jinghuanjs.paths
      });
    } catch (e) {
      return false;
    }
  }

  requireMid(item) {
    let socket = "";

    let jhMid = _path.default.join(jinghuanjs.JH_PATH, 'socket', item.handle);

    let appMid = _path.default.join(jinghuanjs.APP_PATH, 'common', 'socket', item.handle);

    let nodeMid = 'jinghuan-socket-' + item.handle;

    if (_fsExtra.default.pathExistsSync(jhMid)) {
      socket = jhMid;
    } else if (_fsExtra.default.pathExistsSync(appMid)) {
      socket = appMid;
    } else if (this.checkMid(nodeMid)) {
      socket = this.checkMid(nodeMid);
    }

    let cache = require.cache[socket];
    let handle = null;

    if (!cache) {
      handle = require(socket);

      if (handle) {
        item[RUN] = handle(item.options, jinghuanjs.app);
      }
    }
  }

  parse(sockets = []) {
    return sockets.map(item => {
      if ((0, _isString.default)(item)) {
        return {
          handle: item
        };
      }

      if ((0, _isFunction.default)(item)) {
        return {
          handle: () => item
        };
      }

      return item;
    }).filter(item => {
      return !('enable' in item) || item.enable;
    }).map(item => {
      const match = this.createRegexp(item.match);
      const ignore = this.createRegexp(item.ignore);

      if (jinghuanjs.source === 'app') {
        this.requireMid(item);
      }

      return (ctx, next) => {
        if ((0, _isString.default)(item.handle)) {
          this.requireMid(item);
        } else {
          if (!(0, _isFunction.default)(item[RUN])) {
            item[RUN] = item.handle(item.options, jinghuanjs.app);
          }
        }

        if (match && !this.checkMatch(match, ctx) || ignore && this.checkMatch(ignore, ctx)) {
          return next();
        }

        return item[RUN](ctx, next);
      };
    });
  }

  load() {
    let sockets = jinghuanjs.config('socket') || [];
    let ms = sockets.map((v, k) => {
      return v.handle;
    });
    (0, _define.default)('sockets', ms, jinghuan);
    return this.parse(sockets);
  }

}

var _default = new Middleware();

exports.default = _default;
module.exports = exports.default;