"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _assert = _interopRequireDefault(require("assert"));

var _url = _interopRequireDefault(require("url"));

var _onFinished = _interopRequireDefault(require("on-finished"));

var _destroy = _interopRequireDefault(require("destroy"));

var _events = _interopRequireDefault(require("../core/events"));

var _get = _interopRequireDefault(require("lodash/get"));

var _ms = _interopRequireDefault(require("jinghuan-helper/ms"));

var _isArray = _interopRequireDefault(require("lodash/isArray"));

var _mime = _interopRequireDefault(require("mime"));

var _isString = _interopRequireDefault(require("lodash/isString"));

var _pathExtra = _interopRequireDefault(require("path-extra"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const PARAM = Symbol('context-param');
const POST = Symbol('context-post');
const FILE = Symbol('context-file');
const EVENTS = Symbol('context-events');
var _default = {
  get userAgent() {
    return this.header['user-agent'];
  },

  get isGet() {
    return this.method === 'GET';
  },

  get isPost() {
    return this.method === 'POST';
  },

  referer(onlyHost) {
    return this.referrer(onlyHost);
  },

  referrer(onlyHost) {
    const referrer = this.header['referer'];

    if (!referrer || !onlyHost) {
      return referrer;
    }

    return _url.default.parse(referrer).hostname;
  },

  isMethod(method) {
    return this.method === method.toUpperCase();
  },

  isAjax(method) {
    if (method && !this.isMethod(method)) {
      return false;
    }

    return this.header['x-requested-with'] === 'XMLHttpRequest';
  },

  expires(time) {
    time = (0, _ms.default)(time);
    const date = new Date(Date.now() + time);
    this.set('Cache-Control', `max-age=${time}`);
    this.set('Expires', date.toUTCString());
  },

  config(name, value) {
    return jinghuanjs.config(name, value);
  },

  param(name) {
    if (!this[PARAM]) {
      this[PARAM] = Object.assign({}, this.request._query || this.request.query || {});
    }

    if (name == null) {
      return this[PARAM];
    }

    return (0, _get.default)(this[PARAM], name, null);
  },

  post(name) {
    if (!this[POST]) {
      if ((0, _isString.default)(this.request.body && this.request.body.post)) {
        this[POST] = this.request.body && this.request.body.post;
      } else {
        this[POST] = Object.assign({}, this.request.body && this.request.body.post || {});
      }
    }

    if (name == null) {
      return this[POST];
    }

    return (0, _get.default)(this[POST], name, null);
  },

  file(name) {
    if (!this[FILE]) {
      this[FILE] = Object.assign({}, this.request.body && this.request.body.file || {});
    }

    if (name == null) {
      return this[FILE];
    }

    return (0, _get.default)(this[FILE], name, null);
  },

  cookie(name, value, options = {}) {
    if (name == null && value == null) {
      return this.cookies.get();
    }

    if (value == null) {
      return this.cookies.get(name);
    }

    this.cookies.set(name, value, options);
  },

  download(filepath, filename) {
    filename = filename || _pathExtra.default.basename(filepath);
    (0, _assert.default)(filepath, 'filepath can not be empty');
    const contentType = this.response.get('Content-Type');

    if (!contentType) {
      this.type = _pathExtra.default.extname(filename);
    }

    const contentDisposition = this.response.get('Content-Disposition');

    if (!contentDisposition) {
      this.attachment(filename);
    }

    const stream = _fsExtra.default.createReadStream(filepath);

    this.body = stream;
    (0, _onFinished.default)(this.res, () => {
      (0, _destroy.default)(stream);
    });
    return false;
  },

  upload(file, dir, filename, filter, size) {
    let config = jinghuanjs.config('upload');

    let type = _mime.default.getExtension(file.type);

    filter = filter || config.filter;
    size = size || config.size;

    if ((0, _isArray.default)(filter) && filter.indexOf(type) == -1 && filter.indexOf(file.type) == -1) {
      throw new Error('文件类型错误');
    }

    if (size < file.size) {
      throw new Error('文件大小错误');
    }

    try {
      let extname = _pathExtra.default.extname(file.name);

      let pathname = '';

      if (dir.startsWith(config.path)) {
        pathname = dir;
      } else {
        pathname = _pathExtra.default.join(config.path, dir);
      }

      filename = filename + extname;

      let savepath = _pathExtra.default.join(pathname, filename);

      _fsExtra.default.moveSync(file.path, savepath, {
        overwrite: true
      });

      return {
        pathname,
        filename
      };
    } catch (e) {
      jinghuanjs.logger.error(e);
    }

    return false;
  },

  get events() {
    if (!this[EVENTS]) {
      this[EVENTS] = new _events.default();
    }

    return this[EVENTS];
  },

  get db() {
    return (...args) => {
      let ret = jinghuanjs.db(...args);

      try {
        return ret.logger(this.logger('DB'));
      } catch (e) {
        return ret;
      }
    };
  },

  session(...args) {},

  cache(...args) {
    return jinghuanjs.cache(...args);
  },

  display(file = null, data, option = {}) {
    option = option || {};

    if (file == null) {
      file = this.action;
    }

    let {
      module,
      controller,
      action
    } = this;
    option.module = option.module || module;
    option.controller = option.controller || controller;
    option.action = option.action || action;
    option.root = option.root || _pathExtra.default.join(jinghuanjs.APP_PATH, module, 'tpl', controller);
    this.body = jinghuanjs.tpl(file, data, option);
  },

  logger: jinghuanjs.logger
};
exports.default = _default;
module.exports = exports.default;