"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _debug = _interopRequireDefault(require("debug"));

var _isString = _interopRequireDefault(require("lodash/isString"));

var _isObject = _interopRequireDefault(require("lodash/isObject"));

var _isSymbol = _interopRequireDefault(require("lodash/isSymbol"));

var _get = _interopRequireDefault(require("lodash/get"));

var _pathExtra = _interopRequireDefault(require("path-extra"));

var _sprintf = _interopRequireDefault(require("locutus/php/strings/sprintf"));

var _isArray = _interopRequireDefault(require("lodash/isArray"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = (0, _debug.default)('JH:extend/controller');
var _default = {
  get body() {
    return this.ctx.body;
  },

  set body(value) {
    this.ctx.body = value;
  },

  get ip() {
    return this.ctx.ip;
  },

  get ips() {
    return this.ctx.ips;
  },

  get status() {
    return this.ctx.status;
  },

  set status(status) {
    this.ctx.status = status;
  },

  get type() {
    return this.ctx.type;
  },

  set type(contentType) {
    this.ctx.type = contentType;
  },

  get userAgent() {
    return this.ctx.userAgent;
  },

  get method() {
    return this.ctx.method;
  },

  get isGet() {
    return this.ctx.isGet;
  },

  get isPost() {
    return this.ctx.isPost;
  },

  get config() {
    return function (name, value) {
      return jinghuanjs.config(name, value);
    };
  },

  get isMethod() {
    return function (method) {
      return this.ctx.isMethod(method);
    };
  },

  get isAjax() {
    return function (method) {
      return this.ctx.isAjax(method);
    };
  },

  get success() {
    return function (data, message) {
      return this.ctx.success(data, message);
    };
  },

  get fail() {
    return function (errno, errmsg, data) {
      return this.ctx.fail(errno, errmsg, data);
    };
  },

  get expires() {
    return function (time) {
      return this.ctx.expires(time);
    };
  },

  get param() {
    return function (name, value) {
      return this.ctx.param(name, value);
    };
  },

  get post() {
    return function (name, value) {
      return this.ctx.post(name, value);
    };
  },

  get file() {
    return function (name, value) {
      return this.ctx.file(name, value);
    };
  },

  get cookie() {
    return function (name, value, options) {
      return this.ctx.cookie(name, value, options);
    };
  },

  get header() {
    return function (name, value) {
      if (value === undefined && (0, _isString.default)(name)) {
        return this.ctx.header[name];
      }

      if (this.ctx.res.headersSent) {
        log(`headers has already sent, url: ${this.ctx.url}`);
        return;
      }

      if (value !== undefined) {
        return this.ctx.set(name, value);
      }

      if ((0, _isObject.default)(name)) {
        return this.ctx.set(name);
      }
    };
  },

  get referrer() {
    return function (onlyHost) {
      return this.ctx.referer(onlyHost);
    };
  },

  get referer() {
    return function (onlyHost) {
      return this.ctx.referer(onlyHost);
    };
  },

  get redirect() {
    return function (url, alt) {
      this.ctx.redirect(url, alt);
    };
  },

  get download() {
    return function (filepath, filename) {
      return this.ctx.download(filepath, filename);
    };
  },

  get upload() {
    return function (file, dir, filename, filter, size) {
      return this.ctx.upload(file, dir, filename, filter, size);
    };
  },

  get session() {
    return function (name, value, options) {
      return this.ctx.session(name, value, options);
    };
  },

  get db() {
    return this.ctx.db;
  },

  get sql() {
    return function (type, name, args) {
      if (!(0, _isSymbol.default)(type)) {
        args = name;
        name = type;
        type = Symbol('default');
      }

      let sql = (0, _get.default)(jinghuanjs.sql, `${this.ctx.module}.${name}`, name);

      if ((0, _isArray.default)(args)) {
        sql = (0, _sprintf.default)(sql, ...args);
      }

      return this.db(type).sql(sql);
    };
  },

  get cache() {
    return function (name, content, expire) {
      return this.ctx.cache(name, content, expire);
    };
  },

  get display() {
    return function (file, data, option) {
      this.ctx.display(file, data, option);
    };
  },

  get callback() {
    return function (data = null, extend = null, msg = null, status = 0) {
      this.body = {
        data,
        extend,
        msg,
        status
      };
    };
  }

};
exports.default = _default;
module.exports = exports.default;