"use strict";

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _findRoot = _interopRequireDefault(require("find-root"));

var _isString = _interopRequireDefault(require("lodash/isString"));

var _appliaction = _interopRequireDefault(require("./appliaction"));

var _utils = require("./utils");

var _jinghuanLogger = _interopRequireDefault(require("jinghuan-logger"));

var _child_process = require("child_process");

var _isNumber = _interopRequireDefault(require("locutus/isNumber"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = function (options) {
  if (options.debug) {
    if ((0, _isNumber.default)(options.debug)) require('inspector').open(options.debug);else require('inspector').open(9228);
  }

  let appRootPath;

  try {
    appRootPath = options.ROOT_PATH || (0, _findRoot.default)(process.cwd());
  } catch (e) {
    console.error(`"${appRootPath}" Not the nodejs project directory`);
    process.exit(0);
  }

  let rootPath = (0, _findRoot.default)(__filename);
  let filename = process.mainModule.filename;
  options.source = options.source || 'src';
  options.host = options.host || [];
  if ((0, _isString.default)(options.host)) options.host = [options.host];
  options.ROOT_PATH = options.ROOT_PATH || appRootPath;
  options.env = options.env || _path.default.basename(filename, '.js');
  options.LOG_PATH = options.LOG_PATH || _path.default.join(options.ROOT_PATH, 'logs', options.env);

  _jinghuanLogger.default.setOptions({
    log_file_path: options.LOG_PATH
  });

  options.port = options.port || 8320;
  options.watcher = false;
  options.modules = options.modules || [options.env];
  options.paths = [appRootPath, rootPath];
  options.workers = options.workers || 0;
  options.socket = options.socket || false;
  options.JH_PATH = _path.default.join(rootPath, 'lib');

  if (options.ssl) {
    options.ssl = {
      key: options.ssl.key || _fs.default.readFileSync(_path.default.join(__dirname, 'file/private.pem'), 'utf8'),
      cert: options.ssl.cert || _fs.default.readFileSync(_path.default.join(__dirname, 'file/file.crt'), 'utf8'),
      port: options.ssl.port || 443
    };
  }

  if (options.source === 'src') {
    options.watcher = true;

    try {
      let exec = (0, _child_process.execSync)("jinghuan-cli root").toString();

      if (exec) {
        let cliPath = exec.trim();

        try {
          require(_path.default.join(cliPath, 'register.js'))({
            ignore: [function (file) {
              if (/node_modules/.test(file)) return true;else if (file.startsWith(options.ROOT_PATH)) {
                return false;
              }
              return true;
            }]
          });
        } catch (e) {
          console.error(e);
        }
      } else {
        throw new Error('run npm i jinghuan-cli -g');
      }
    } catch (e) {
      _jinghuanLogger.default.error(e.message);

      process.exit(0);
    }
  }

  options.APP_PATH = _path.default.join(options.ROOT_PATH, options.source);
  let app = new _appliaction.default(options);
  app.run();
};