"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _jinghuanConfig = _interopRequireDefault(require("jinghuan-config"));

var _loader = _interopRequireDefault(require("./core/loader"));

var _define = _interopRequireDefault(require("jinghuan-helper/define"));

var _cluster = _interopRequireDefault(require("cluster"));

var _koaCompose = _interopRequireDefault(require("koa-compose"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Loaders {
  constructor() {
    this.loader = new _loader.default();
  }

  loadData() {
    this.loader.loadEvents();
    (0, _define.default)('controllers', this.loader.loadController(), jinghuan);
    (0, _define.default)('sql', this.loader.loadSql(), jinghuan);
    (0, _define.default)('validate', this.loader.loadValidate(), jinghuan);
  }

  loadMiddleware() {
    const middlewares = this.loader.loadMiddleware();
    jinghuanjs.app.use((0, _koaCompose.default)(middlewares));
  }

  loadSocket() {
    const sockets = this.loader.loadSocket();
    sockets.forEach(socket => {
      jinghuanjs.app.ws.use(socket);
    });
  }

  loadExtend() {
    let {
      JH_PATH,
      ROOT_PATH
    } = jinghuan;
    let extend = this.loader.loadExtend(JH_PATH);
    let list = [['jinghuan', jinghuanjs], ['jinghuanjs', jinghuanjs], ['application', jinghuanjs.app], ['context', jinghuanjs.app.context], ['controller', jinghuanjs.Controller.prototype]];
    list.forEach(val => {
      if (!extend[val[0]]) {
        return;
      }

      _loader.default.extend(val[1], extend[val[0]]);
    });
    extend = this.loader.loadExtend(_path.default.join(ROOT_PATH, jinghuanjs.source, 'common'));
    list.forEach(val => {
      if (!extend[val[0]]) {
        return;
      }

      _loader.default.extend(val[1], extend[val[0]]);
    });
  }

  loadConfig() {
    let {
      JH_PATH,
      CONFIG_PATH
    } = jinghuanjs;

    const jinghuanjsConfig = _path.default.join(JH_PATH, 'config');

    (0, _define.default)('config', (0, _jinghuanConfig.default)([jinghuanjsConfig, CONFIG_PATH]), jinghuanjs);
  }

  load(isAll = true) {
    this.loadConfig();

    if (isAll) {
      this.loadExtend();
      this.loadData();
      this.loadMiddleware();
      this.loadSocket();
    }
  }

}

;
var _default = Loaders;
exports.default = _default;
module.exports = exports.default;